import { defineStore, type StoreDefinition } from 'pinia'
import { io, Socket } from 'socket.io-client';
import {useCookies} from '@vueuse/integrations/useCookies'
import config from '@/config';

export const useSocketStore: StoreDefinition<any> = defineStore('socket', {
  state: () => ({
    connection: null as Socket | null,
    character: null as any,
  }),
  getters: {
    getConnection: (state: any) => state.connection,
    getCharacter: (state: any) => state.character,
  },
  actions: {
    setupSocketConnection() {
      this.connection = io(config.server_endpoint, {
        withCredentials: true,
        transports: ['websocket'],
        reconnectionAttempts: 5,
      });

      // Let the server know the user is logged in
      this.connection.emit('login');

      // When we can't reconnect, disconnect
      this.connection.on('reconnect_failed', () => {
        console.log("Reconnect failed")
        this.disconnectSocket();
      })
    },
    disconnectSocket() {
      if (!this.connection) return;

      this.connection.disconnect();
      this.connection = null;
      this.character = null;

      useCookies().remove('token');
    },
  }
});

/**
 * Resources:
 * https://www.dynetisgames.com/2017/03/06/how-to-make-a-multiplayer-online-game-with-phaser-socket-io-and-node-js/
 * https://runthatline.com/pinia-watch-state-getters-inside-vue-components/
 * https://pinia.vuejs.org/
 */